/* Automatically generated file, do not edit */
#ifndef _h_vi_c
#define _h_vi_c
libedit_private el_action_t	vi_paste_next (EditLine *, wint_t);
libedit_private el_action_t	vi_paste_prev (EditLine *, wint_t);
libedit_private el_action_t	vi_prev_big_word (EditLine *, wint_t);
libedit_private el_action_t	vi_prev_word (EditLine *, wint_t);
libedit_private el_action_t	vi_next_big_word (EditLine *, wint_t);
libedit_private el_action_t	vi_next_word (EditLine *, wint_t);
libedit_private el_action_t	vi_change_case (EditLine *, wint_t);
libedit_private el_action_t	vi_change_meta (EditLine *, wint_t);
libedit_private el_action_t	vi_insert_at_bol (EditLine *, wint_t);
libedit_private el_action_t	vi_replace_char (EditLine *, wint_t);
libedit_private el_action_t	vi_replace_mode (EditLine *, wint_t);
libedit_private el_action_t	vi_substitute_char (EditLine *, wint_t);
libedit_private el_action_t	vi_substitute_line (EditLine *, wint_t);
libedit_private el_action_t	vi_change_to_eol (EditLine *, wint_t);
libedit_private el_action_t	vi_insert (EditLine *, wint_t);
libedit_private el_action_t	vi_add (EditLine *, wint_t);
libedit_private el_action_t	vi_add_at_eol (EditLine *, wint_t);
libedit_private el_action_t	vi_delete_meta (EditLine *, wint_t);
libedit_private el_action_t	vi_end_big_word (EditLine *, wint_t);
libedit_private el_action_t	vi_end_word (EditLine *, wint_t);
libedit_private el_action_t	vi_undo (EditLine *, wint_t);
libedit_private el_action_t	vi_command_mode (EditLine *, wint_t);
libedit_private el_action_t	vi_zero (EditLine *, wint_t);
libedit_private el_action_t	vi_delete_prev_char (EditLine *, wint_t);
libedit_private el_action_t	vi_list_or_eof (EditLine *, wint_t);
libedit_private el_action_t	vi_kill_line_prev (EditLine *, wint_t);
libedit_private el_action_t	vi_search_prev (EditLine *, wint_t);
libedit_private el_action_t	vi_search_next (EditLine *, wint_t);
libedit_private el_action_t	vi_repeat_search_next (EditLine *, wint_t);
libedit_private el_action_t	vi_repeat_search_prev (EditLine *, wint_t);
libedit_private el_action_t	vi_next_char (EditLine *, wint_t);
libedit_private el_action_t	vi_prev_char (EditLine *, wint_t);
libedit_private el_action_t	vi_to_next_char (EditLine *, wint_t);
libedit_private el_action_t	vi_to_prev_char (EditLine *, wint_t);
libedit_private el_action_t	vi_repeat_next_char (EditLine *, wint_t);
libedit_private el_action_t	vi_repeat_prev_char (EditLine *, wint_t);
libedit_private el_action_t	vi_match (EditLine *, wint_t);
libedit_private el_action_t	vi_undo_line (EditLine *, wint_t);
libedit_private el_action_t	vi_to_column (EditLine *, wint_t);
libedit_private el_action_t	vi_yank_end (EditLine *, wint_t);
libedit_private el_action_t	vi_yank (EditLine *, wint_t);
libedit_private el_action_t	vi_comment_out (EditLine *, wint_t);
libedit_private el_action_t	vi_alias (EditLine *, wint_t);
libedit_private el_action_t	vi_to_history_line (EditLine *, wint_t);
libedit_private el_action_t	vi_histedit (EditLine *, wint_t);
libedit_private el_action_t	vi_history_word (EditLine *, wint_t);
libedit_private el_action_t	vi_redo (EditLine *, wint_t);
#endif /* _h_vi_c */
